/*
 * Copyright (C) 2012-2013 The Dojo Foundation
 * 
 * Copyright (C) 2012-2013 Rex Young
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package com.fastmessenger.impl;

import java.util.concurrent.ExecutorService;

import org.eclipse.swt.widgets.Display;

import com.fastmessenger.model.IMessenger;

/**
 * A messenger object with extra features to support GUI receivers that should be executed in a
 * corresponding UI thread.
 * 
 * @author Rex Young
 * 
 * @see IMessenger
 * @see Messenger
 */
public class GuiMessenger extends Messenger {

    private Display swtDisplay;

    /**
     * @see Messenger#Messenger()
     */
    public GuiMessenger () {
        super ();
    }

    /**
     * @param executorService
     * 
     * @see Messenger#Messenger(ExecutorService)
     */
    public GuiMessenger (ExecutorService executorService) {
        super (executorService);
    }

    public Display getSwtDisplay () {
        return swtDisplay;
    }

    public void setSwtDisplay (Display swtDisplay) {
        this.swtDisplay = swtDisplay;
    }

    /**
     * Creates a receiver (namely an active object with incoming ports) and associates it with an
     * implementation object.
     * <p>
     * All requests for this active object will be executed in JavaFX UI thread.
     * 
     * @param implObj
     *            an implementation object
     * @param receiver
     *            name of a receiver to be created
     * @param ports
     *            names of the receiver's incoming ports
     * 
     * @see Messenger#registerReceiver(Object, String, String...)
     */
    synchronized public void registerJavaFxReceiver (Object implObj, String receiver, String... ports) {
        Object[] implObjs = new Object[] { implObj };
        JavaFxDispatcher dispatcher = new JavaFxDispatcher (this, implObjs);

        registerReceiverImpl (receiver, ports, implObjs, dispatcher);
    }

    /**
     * Creates a receiver (namely an active object with incoming ports) and associates it with an
     * implementation object.
     * <p>
     * All requests for this active object will be executed in Swing UI thread.
     * 
     * @param implObj
     *            an implementation object
     * @param receiver
     *            name of a receiver to be created
     * @param ports
     *            names of the receiver's incoming ports
     * 
     * @see Messenger#registerReceiver(Object, String, String...)
     */
    synchronized public void registerSwingReceiver (Object implObj, String receiver, String... ports) {
        Object[] implObjs = new Object[] { implObj };
        SwingDispatcher dispatcher = new SwingDispatcher (this, implObjs);

        registerReceiverImpl (receiver, ports, implObjs, dispatcher);
    }

    /**
     * Creates a receiver (namely an active object with incoming ports) and associates it with an
     * implementation object.
     * <p>
     * All requests for this active object will be executed in SWT UI thread.
     * 
     * @param implObj
     *            an implementation object
     * @param receiver
     *            name of a receiver to be created
     * @param ports
     *            names of the receiver's incoming ports
     * 
     * @see Messenger#registerReceiver(Object, String, String...)
     */
    synchronized public void registerSwtReceiver (Object implObj, String receiver, String... ports) {
        Object[] implObjs = new Object[] { implObj };
        SwtDispatcher dispatcher = new SwtDispatcher (swtDisplay, this, implObjs);

        registerReceiverImpl (receiver, ports, implObjs, dispatcher);
    }

}
